<?php

/**
 * Copyright (c) D3 Data Development (Inh. Thomas Dartsch)
 *
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 *
 * https://www.d3data.de
 *
 * @copyright (C) D3 Data Development (Inh. Thomas Dartsch)
 * @author    D3 Data Development - Daniel Seifert <info@shopmodule.com>
 * @link      https://www.oxidmodule.com
 */

declare(strict_types=1);

namespace D3\Usermanager\tests\integration\Requirements;

use D3\ModCfg\Application\Model\Exception\d3_cfg_mod_exception;
use D3\ModCfg\Application\Model\Exception\d3ShopCompatibilityAdapterException;
use D3\Usermanager\Application\Model\d3usermanager;
use Exception;
use OxidEsales\Eshop\Application\Model\UserList;
use OxidEsales\Eshop\Core\Exception\DatabaseConnectionException;
use OxidEsales\Eshop\Core\Exception\DatabaseErrorException;
use OxidEsales\Eshop\Core\Exception\StandardException;
use PHPUnit\Framework\MockObject\MockObject;
 
class requirementExecuteMethodFilterTest extends d3RequirementIntegrationTestCase
{
    public $sManagerId = 'managerTestId';
    public $aUserIdList = [
        'userTestIdNo1',
        'userTestIdNo2',
    ];

    /**
     * Set up fixture.
     * @throws Exception
     */
    public function setUp(): void
    {
        parent::setUp();

        $this->createTestData();
    }

    /**
     * Tear down fixture.
     */
    public function tearDown(): void
    {
        $this->cleanTestData();

        parent::tearDown();
    }

    /**
     * @throws Exception
     */
    public function createTestData()
    {
        $this->createManager(
            $this->sManagerId
        );

        $this->createUser(
            $this->aUserIdList[0],
            [
                'oxcompany' => __CLASS__,
                'oxcustnr'      => 'TestBillNo1',
            ]
        );

        $this->createUser(
            $this->aUserIdList[1],
            [
                'oxcompany' => __CLASS__,
                'oxcustnr'      => '0',
            ]
        );
    }

    public function cleanTestData()
    {
        $this->deleteManager($this->sManagerId);

        foreach ($this->aUserIdList as $sUserId) {
            $this->deleteUser($sUserId);
        }
    }

    /**
     * @return d3usermanager
     * @throws Exception
     */
    public function getConfiguredManager(): d3usermanager
    {
        $oManager = $this->getManagerMock($this->sManagerId);

        $oManager->setValue('blItemExecute', true);
        $oManager->setValue('blCheckExecuteMethod_status', true);
        $oManager->setValue('sRequirementExecuteMethod_name', 'testChangeUserList');

        return $oManager;
    }

    /**
     * @test
     * @throws DatabaseConnectionException
     * @throws DatabaseErrorException
     * @throws StandardException
     * @throws d3ShopCompatibilityAdapterException
     * @throws d3_cfg_mod_exception
     * @throws Exception
     */
    public function requirementsSelectsRightUsers()
    {
        $oListGenerator = $this->getListGenerator($this->getConfiguredManager());

        $definitions = d3GetOxidDIC()->getDefinitions();

        /** @var UserList|MockObject $oListMock */
        $oListMock = $this->getMockBuilder(UserList::class)
            ->addMethods(['testChangeUserList'])
            ->getMock();
        $oListMock->expects($this->once())->method('testChangeUserList')->willReturn(null);
        d3GetOxidDIC()->set('d3ox.usermanager.'.UserList::class, $oListMock);

        $oUserList = $oListGenerator->getConcernedItems();

        d3GetOxidDIC()->reset();
        d3GetOxidDIC()->setDefinitions($definitions);

        $this->assertTrue(
            $oUserList->count() >= 2
            && $oUserList->offsetExists($this->aUserIdList[0])
            && $oUserList->offsetExists($this->aUserIdList[1])
        );
    }
}
