<?php

/**
 * Copyright (c) D3 Data Development (Inh. Thomas Dartsch)
 *
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 *
 * https://www.d3data.de
 *
 * @copyright (C) D3 Data Development (Inh. Thomas Dartsch)
 * @author    D3 Data Development - Daniel Seifert <info@shopmodule.com>
 * @link      https://www.oxidmodule.com
 */

declare(strict_types=1);

namespace D3\Usermanager\tests\integration\Requirements;

use D3\ModCfg\Application\Model\Exception\d3_cfg_mod_exception;
use D3\ModCfg\Application\Model\Exception\d3ShopCompatibilityAdapterException;
use D3\Usermanager\Application\Model\d3usermanager;
use Exception;
use OxidEsales\Eshop\Core\Exception\DatabaseConnectionException;
use OxidEsales\Eshop\Core\Exception\DatabaseErrorException;
use OxidEsales\Eshop\Core\Exception\StandardException;
 
class requirementNewsletterFilterTest extends d3RequirementIntegrationTestCase
{
    public $sManagerId = 'managerTestId';
    public $aUserIdList = [
        'userTestIdNo1',
        'userTestIdNo2',
        'userTestIdNo3',
    ];
    public $aNewsSubscribedIdList = [
        'newsletterIdNo1',
        'newsletterIdNo2',
    ];

    /**
     * Set up fixture.
     * @throws Exception
     */
    public function setUp(): void
    {
        parent::setUp();

        $this->createTestData();
    }

    /**
     * Tear down fixture.
     */
    public function tearDown(): void
    {
        $this->cleanTestData();

        parent::tearDown();
    }

    /**
     * @throws Exception
     */
    public function createTestData()
    {
        $this->createManager(
            $this->sManagerId
        );

        $this->createUser(
            $this->aUserIdList[0],
            [
                'oxcompany' => __CLASS__,
            ]
        );

        $this->createBaseModelObject(
            'oxnewssubscribed',
            $this->aNewsSubscribedIdList[0],
            [
                'oxuserid'    => $this->aUserIdList[0],
                'oxdboptin'   => '1',
            ]
        );

        $this->createUser(
            $this->aUserIdList[1],
            [
                'oxcompany' => __CLASS__,
            ]
        );

        $this->createBaseModelObject(
            'oxnewssubscribed',
            $this->aNewsSubscribedIdList[1],
            [
                'oxuserid'    => $this->aUserIdList[1],
                'oxdboptin'    => '2',
            ]
        );

        $this->createUser(
            $this->aUserIdList[2],
            [
                'oxcompany' => __CLASS__,
            ]
        );
    }

    public function cleanTestData()
    {
        $this->deleteManager($this->sManagerId);

        foreach ($this->aUserIdList as $sUserId) {
            $this->deleteUser($sUserId);
        }

        foreach ($this->aNewsSubscribedIdList as $sNewsletterId) {
            $this->deleteBaseModelObject('oxnewssubscribed', $sNewsletterId);
        }
    }

    /**
     * @return d3usermanager
     * @throws Exception
     */
    public function getConfiguredManagerSet()
    {
        $oManager = $this->getManagerMock($this->sManagerId);

        $oManager->setValue('blCheckNewsletterUser_status', true);
        $oManager->setValue('sCheckNewsletterUserType', 'set');

        return $oManager;
    }

    /**
     * @return d3usermanager
     * @throws Exception
     */
    public function getConfiguredManagerNotSet()
    {
        $oManager = $this->getManagerMock($this->sManagerId);

        $oManager->setValue('blCheckNewsletterUser_status', true);
        $oManager->setValue('sCheckNewsletterUserType', 'notset');

        return $oManager;
    }

    /**
     * @test
     * @throws DatabaseConnectionException
     * @throws DatabaseErrorException
     * @throws StandardException
     * @throws d3ShopCompatibilityAdapterException
     * @throws d3_cfg_mod_exception
     * @throws Exception
     */
    public function requirementsSelectsRightUsersSet()
    {
        $oListGenerator = $this->getListGenerator($this->getConfiguredManagerSet());
        $oUserList = $oListGenerator->getConcernedItems();

        $this->assertTrue(
            $oUserList->count() >= 1
            && $oUserList->offsetExists($this->aUserIdList[0])
            && false == $oUserList->offsetExists($this->aUserIdList[1])
            && false == $oUserList->offsetExists($this->aUserIdList[2])
        );
    }

    /**
     * @test
     * @throws DatabaseConnectionException
     * @throws DatabaseErrorException
     * @throws StandardException
     * @throws d3ShopCompatibilityAdapterException
     * @throws d3_cfg_mod_exception
     * @throws Exception
     */
    public function requirementsSelectsRightUsersNotSet()
    {
        $oListGenerator = $this->getListGenerator($this->getConfiguredManagerNotSet());
        $oUserList = $oListGenerator->getConcernedItems();

        $this->assertTrue(
            $oUserList->count() >= 2
            && false == $oUserList->offsetExists($this->aUserIdList[0])
            && $oUserList->offsetExists($this->aUserIdList[1])
            && $oUserList->offsetExists($this->aUserIdList[2])
        );
    }
}
