<?php

/**
 * Copyright (c) D3 Data Development (Inh. Thomas Dartsch)
 *
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 *
 * https://www.d3data.de
 *
 * @copyright (C) D3 Data Development (Inh. Thomas Dartsch)
 * @author    D3 Data Development - Daniel Seifert <info@shopmodule.com>
 * @link      https://www.oxidmodule.com
 */

declare(strict_types=1);

namespace D3\Usermanager\tests\integration\Requirements;

use D3\ModCfg\Application\Model\Exception\d3_cfg_mod_exception;
use D3\ModCfg\Application\Model\Exception\d3ShopCompatibilityAdapterException;
use D3\Usermanager\Application\Model\d3usermanager;
use Exception;
use OxidEsales\Eshop\Core\Exception\DatabaseConnectionException;
use OxidEsales\Eshop\Core\Exception\DatabaseErrorException;
use OxidEsales\Eshop\Core\Exception\StandardException;
 
class requirementNotInGroupFilterTest extends d3RequirementIntegrationTestCase
{
    public $sManagerId = 'managerTestId';
    public $aUserIdList = [
        'userTestIdNo1',
        'userTestIdNo2',
        'userTestIdNo3',
    ];
    public $aGroupAssignIdList = [
        'groupAssignIdNo1',
        'groupAssignIdNo2',
        'groupAssignIdNo3',
    ];
    public $aGroupIdList = [
        'sGroupId1Pass',
        'sGroupId2Pass',
        'sGroupId3Pass',
    ];

    /**
     * Set up fixture.
     * @throws Exception
     */
    public function setUp(): void
    {
        parent::setUp();

        $this->createTestData();
    }

    /**
     * Tear down fixture.
     */
    public function tearDown(): void
    {
        $this->cleanTestData();

        parent::tearDown();
    }

    /**
     * @throws Exception
     */
    public function createTestData()
    {
        $this->createManager(
            $this->sManagerId
        );

        $this->createUser(
            $this->aUserIdList[0],
            [
                'oxcompany' => __CLASS__,
            ]
        );

        $this->createBaseModelObject(
            'oxobject2group',
            $this->aGroupAssignIdList[0],
            [
                'oxobjectid'    => $this->aUserIdList[0],
                'oxgroupsid'    => 'sGroupId3Pass',
            ]
        );

        $this->createUser(
            $this->aUserIdList[1],
            [
                'oxcompany' => __CLASS__,
            ]
        );

        $this->createBaseModelObject(
            'oxobject2group',
            $this->aGroupAssignIdList[1],
            [
                'oxobjectid'    => $this->aUserIdList[1],
                'oxgroupsid'    => 'sGroupId2Pass',
            ]
        );

        $this->createUser(
            $this->aUserIdList[2],
            [
                'oxcompany' => __CLASS__,
            ]
        );

        $this->createBaseModelObject(
            'oxobject2group',
            $this->aGroupAssignIdList[2],
            [
                'oxobjectid'    => $this->aUserIdList[2],
                'oxgroupsid'    => 'sGroupId1Pass',
            ]
        );

        foreach ($this->aGroupIdList as $sId) {
            $this->createBaseModelObject('oxgroups', $sId);
        }
    }

    public function cleanTestData()
    {
        $this->deleteManager($this->sManagerId);

        foreach ($this->aUserIdList as $sUserId) {
            $this->deleteUser($sUserId);
        }

        foreach ($this->aGroupAssignIdList as $sGroupAssignId) {
            $this->deleteBaseModelObject('oxobject2group', $sGroupAssignId);
        }

        foreach ($this->aGroupIdList as $sId) {
            $this->deleteBaseModelObject('oxgroups', $sId);
        }
    }

    /**
     * @return d3usermanager
     * @throws Exception
     */
    public function getConfiguredManagerSingle()
    {
        $oManager = $this->getManagerMock($this->sManagerId);

        $oManager->setValue('blCheckNotInGroup_status', true);
        $oManager->setValue('aCustNotInGroupId', ['sGroupId1Pass']);

        return $oManager;
    }

    /**
     * @return d3usermanager
     * @throws Exception
     */
    public function getConfiguredManagerMulti()
    {
        $oManager = $this->getManagerMock($this->sManagerId);

        $oManager->setValue('blCheckNotInGroup_status', true);
        $oManager->setValue('aCustNotInGroupId', ['sGroupId1Pass', 'sGroupId2Pass']);

        return $oManager;
    }

    /**
     * @test
     * @throws DatabaseConnectionException
     * @throws DatabaseErrorException
     * @throws StandardException
     * @throws d3ShopCompatibilityAdapterException
     * @throws d3_cfg_mod_exception
     * @throws Exception
     */
    public function requirementsSelectsRightUsersMulti()
    {
        $oListGenerator = $this->getListGenerator($this->getConfiguredManagerMulti());
        $oUserList = $oListGenerator->getConcernedItems();

        $this->assertTrue(
            $oUserList->count() >= 1
            && $oUserList->offsetExists($this->aUserIdList[0])
            && false == $oUserList->offsetExists($this->aUserIdList[1])
            && false == $oUserList->offsetExists($this->aUserIdList[2])
        );
    }

    /**
     * @test
     * @throws DatabaseConnectionException
     * @throws DatabaseErrorException
     * @throws StandardException
     * @throws d3ShopCompatibilityAdapterException
     * @throws d3_cfg_mod_exception
     * @throws Exception
     */
    public function requirementsSelectsRightUsersSingle()
    {
        $oListGenerator = $this->getListGenerator($this->getConfiguredManagerSingle());
        $oUserList = $oListGenerator->getConcernedItems();

        $this->assertTrue(
            $oUserList->count() >= 2
            && $oUserList->offsetExists($this->aUserIdList[0])
            && $oUserList->offsetExists($this->aUserIdList[1])
            && false == $oUserList->offsetExists($this->aUserIdList[2])
        );
    }
}
