<?php

/**
 * Copyright (c) D3 Data Development (Inh. Thomas Dartsch)
 *
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 *
 * https://www.d3data.de
 *
 * @copyright (C) D3 Data Development (Inh. Thomas Dartsch)
 * @author    D3 Data Development - Daniel Seifert <info@shopmodule.com>
 * @link      https://www.oxidmodule.com
 */

declare(strict_types=1);

namespace D3\Usermanager\tests\integration\Requirements;

use D3\ModCfg\Application\Model\Exception\d3_cfg_mod_exception;
use D3\ModCfg\Application\Model\Exception\d3ShopCompatibilityAdapterException;
use D3\Usermanager\Application\Model\d3usermanager;
use Exception;
use OxidEsales\Eshop\Core\Exception\DatabaseConnectionException;
use OxidEsales\Eshop\Core\Exception\DatabaseErrorException;
use OxidEsales\Eshop\Core\Exception\StandardException;
 
class requirementOtherJobTest extends d3RequirementIntegrationTestCase
{
    public $sManagerId = 'managerTestId';
    public $aUserIdList = [
        'userTestIdNo1',
        'userTestIdNo2',
        'userTestIdNo3',
    ];
    public $aAssignIdList = [
        'toUserManagerAssignIdNo1',
        'toUserManagerAssignIdNo2',
    ];

    /**
     * Set up fixture.
     * @throws Exception
     */
    public function setUp(): void
    {
        parent::setUp();

        $this->createTestData();
    }

    /**
     * Tear down fixture.
     */
    public function tearDown(): void
    {
        $this->cleanTestData();

        parent::tearDown();
    }

    /**
     * @throws Exception
     */
    public function createTestData()
    {
        $this->createManager(
            $this->sManagerId
        );

        $this->createUser(
            $this->aUserIdList[0],
            [
                'oxcompany' => __CLASS__,
            ]
        );

        $this->createBaseModelObject(
            'd3user2usermanager',
            $this->aAssignIdList[0],
            [
                'oxuserid'    => $this->aUserIdList[0],
                'oxusermanagerid'    => 'TestJobIdPass',
            ]
        );

        $this->createUser(
            $this->aUserIdList[1],
            [
                'oxcompany' => __CLASS__,
            ]
        );

        $this->createBaseModelObject(
            'd3user2usermanager',
            $this->aAssignIdList[1],
            [
                'oxuserid'    => $this->aUserIdList[1],
                'oxusermanagerid'    => 'TestJobIdNotPass',
            ]
        );

        $this->createUser(
            $this->aUserIdList[2],
            [
                'oxcompany' => __CLASS__,
            ]
        );
    }

    public function cleanTestData()
    {
        $this->deleteManager($this->sManagerId);

        foreach ($this->aUserIdList as $sUserId) {
            $this->deleteUser($sUserId);
        }

        foreach ($this->aAssignIdList as $sAssignId) {
            $this->deleteBaseModelObject('d3user2usermanager', $sAssignId);
        }
    }

    /**
     * @return d3usermanager
     * @throws Exception
     */
    public function getConfiguredManagerNotExec()
    {
        $oManager = $this->getManagerMock($this->sManagerId);

        $oManager->setValue('blCheckOtherJob_status', true);
        $oManager->setValue('sOtherJob_Type', 'notexec');
        $oManager->setValue('sOtherJob_ID', 'TestJobIdPass');

        return $oManager;
    }

    /**
     * @test
     * @throws DatabaseConnectionException
     * @throws DatabaseErrorException
     * @throws StandardException
     * @throws d3ShopCompatibilityAdapterException
     * @throws d3_cfg_mod_exception
     * @throws Exception
     */
    public function requirementsSelectsRightUsersNotExec()
    {
        $oListGenerator = $this->getListGenerator($this->getConfiguredManagerNotExec());
        $oUserList = $oListGenerator->getConcernedItems();

        $this->assertTrue(
            $oUserList->count() >= 2
            && $oUserList->offsetExists($this->aUserIdList[1])
            && $oUserList->offsetExists($this->aUserIdList[2])
            && false == $oUserList->offsetExists($this->aUserIdList[0])
        );
    }

    /**
     * @return d3usermanager
     * @throws Exception
     */
    public function getConfiguredManagerExec()
    {
        $oManager = $this->getManagerMock($this->sManagerId);

        $oManager->setValue('blCheckOtherJob_status', true);
        $oManager->setValue('sOtherJob_Type', 'exec');
        $oManager->setValue('sOtherJob_ID', 'TestJobIdPass');

        return $oManager;
    }

    /**
     * @test
     * @throws DatabaseConnectionException
     * @throws DatabaseErrorException
     * @throws StandardException
     * @throws d3ShopCompatibilityAdapterException
     * @throws d3_cfg_mod_exception
     * @throws Exception
     */
    public function requirementsSelectsRightUsersExec()
    {
        $oListGenerator = $this->getListGenerator($this->getConfiguredManagerExec());
        $oUserList = $oListGenerator->getConcernedItems();

        $this->assertTrue($oUserList->count() >= 1);
        $this->assertTrue($oUserList->offsetExists($this->aUserIdList[0]));
        $this->assertFalse($oUserList->offsetExists($this->aUserIdList[1]));
        $this->assertFalse($oUserList->offsetExists($this->aUserIdList[2]));
    }
}
