<?php

/**
 * Copyright (c) D3 Data Development (Inh. Thomas Dartsch)
 *
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 *
 * https://www.d3data.de
 *
 * @copyright (C) D3 Data Development (Inh. Thomas Dartsch)
 * @author    D3 Data Development - Daniel Seifert <info@shopmodule.com>
 * @link      https://www.oxidmodule.com
 */

declare(strict_types=1);

namespace D3\Usermanager\tests\integration\Requirements;

use D3\ModCfg\Application\Model\Exception\d3_cfg_mod_exception;
use D3\ModCfg\Application\Model\Exception\d3ShopCompatibilityAdapterException;
use D3\Usermanager\Application\Model\d3usermanager;
use Exception;
use OxidEsales\Eshop\Core\Exception\DatabaseConnectionException;
use OxidEsales\Eshop\Core\Exception\DatabaseErrorException;
use OxidEsales\Eshop\Core\Exception\StandardException;
 
class requirementRegisterToTest extends d3RequirementIntegrationTestCase
{
    public $sManagerId = 'managerTestId';
    public $aUserIdList = [
        'userTestIdNo1',
        'userTestIdNo2',
        'userTestIdNo3',
        'userTestIdNo4',
    ];

    /**
     * Set up fixture.
     * @throws Exception
     */
    public function setUp(): void
    {
        parent::setUp();

        $this->createTestData();
    }

    /**
     * Tear down fixture.
     */
    public function tearDown(): void
    {
        $this->cleanTestData();

        parent::tearDown();
    }

    /**
     * @throws Exception
     */
    public function createTestData()
    {
        $this->createManager(
            $this->sManagerId
        );

        $this->createUser(
            $this->aUserIdList[0],
            [
                'oxcompany' => __CLASS__,
                'oxregister'    => date('Y-m-d H:i:s', strtotime(date('Y-m-d H:i:s') . ' -2 year')),
            ]
        );

        $this->createUser(
            $this->aUserIdList[1],
            [
                'oxcompany' => __CLASS__,
                'oxregister'    => date('Y-m-d H:i:s', strtotime(date('Y-m-d H:i:s') . ' -1 year - 1 minute')),
            ]
        );

        $this->createUser(
            $this->aUserIdList[2],
            [
                'oxcompany' => __CLASS__,
                'oxregister'    => date('Y-m-d H:i:s', strtotime(date('Y-m-d H:i:s') . ' -1 year + 1 minute')),
            ]
        );

        $this->createUser(
            $this->aUserIdList[3],
            [
                'oxcompany' => __CLASS__,
            ]
        );
    }

    public function cleanTestData()
    {
        $this->deleteManager($this->sManagerId);

        foreach ($this->aUserIdList as $sUserId) {
            $this->deleteUser($sUserId);
        }
    }

    /**
     * @return d3usermanager
     * @throws Exception
     */
    public function getConfiguredManager()
    {
        $oManager = $this->getManagerMock($this->sManagerId);

        $oManager->setValue('blCheckRegisterUserTo_status', true);
        $oManager->setValue('sRegisterUserToValue', date('Y-m-d H:i:s', strtotime(date('Y-m-d H:i:s') . ' -1 year')));

        return $oManager;
    }

    /**
     * @test
     * @throws DatabaseConnectionException
     * @throws DatabaseErrorException
     * @throws StandardException
     * @throws d3ShopCompatibilityAdapterException
     * @throws d3_cfg_mod_exception
     * @throws Exception
     */
    public function requirementsSelectsRightUsers()
    {
        $oListGenerator = $this->getListGenerator($this->getConfiguredManager());
        $oUserList = $oListGenerator->getConcernedItems();

        $this->assertTrue(
            $oUserList->count() >= 3
            && $oUserList->offsetExists($this->aUserIdList[0])
            && $oUserList->offsetExists($this->aUserIdList[1])
            && false == $oUserList->offsetExists($this->aUserIdList[2])
            && $oUserList->offsetExists($this->aUserIdList[3])
        );
    }
}
