<?php

/**
 * Copyright (c) D3 Data Development (Inh. Thomas Dartsch)
 *
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 *
 * https://www.d3data.de
 *
 * @copyright (C) D3 Data Development (Inh. Thomas Dartsch)
 * @author    D3 Data Development - Daniel Seifert <info@shopmodule.com>
 * @link      https://www.oxidmodule.com
 */

declare(strict_types=1);

namespace D3\Usermanager\tests\integration\Requirements;

use D3\ModCfg\Application\Model\Exception\d3_cfg_mod_exception;
use D3\ModCfg\Application\Model\Exception\d3ShopCompatibilityAdapterException;
use D3\Usermanager\Application\Model\d3usermanager;
use Exception;
use OxidEsales\Eshop\Core\Exception\DatabaseConnectionException;
use OxidEsales\Eshop\Core\Exception\DatabaseErrorException;
use OxidEsales\Eshop\Core\Exception\StandardException;
 
class requirementUserbasketFromTest extends d3RequirementIntegrationTestCase
{
    public $sManagerId = 'managerTestId';
    public $aUserIdList = [
        'userTestIdNo1',
        'userTestIdNo2',
        'userTestIdNo3',
        'userTestIdNo4',
        'userTestIdNo5',
    ];
    public $aUserBasketIdList = [
        'userBasketIdNo1',
        'userBasketIdNo2',
        'userBasketIdNo3',
        'userBasketIdNo4',
    ];

    /**
     * Set up fixture.
     * @throws Exception
     */
    public function setUp(): void
    {
        parent::setUp();

        $this->createTestData();
    }

    /**
     * Tear down fixture.
     */
    public function tearDown(): void
    {
        $this->cleanTestData();

        parent::tearDown();
    }

    /**
     * @throws Exception
     */
    public function createTestData()
    {
        $this->createManager(
            $this->sManagerId
        );

        $this->createUser(
            $this->aUserIdList[0],
            [
                'oxcompany' => __CLASS__,
            ]
        );

        $this->createBaseModelObject(
            'oxuserbaskets',
            $this->aUserBasketIdList[0],
            [
                'oxuserid'    => $this->aUserIdList[0],
                'oxtitle'     => 'savedbasket',
                'oxupdate'     => strtotime(date('Y-m-d')),
            ]
        );

        $this->createUser(
            $this->aUserIdList[1],
            [
                'oxcompany' => __CLASS__,
            ]
        );

        $this->createBaseModelObject(
            'oxuserbaskets',
            $this->aUserBasketIdList[1],
            [
                'oxuserid'    => $this->aUserIdList[1],
                'oxtitle'     => 'savedbasket',
                'oxupdate'     => strtotime(date('Y-m-d') . ' -1 year + 1 minute'),
            ]
        );

        $this->createUser(
            $this->aUserIdList[2],
            [
                'oxcompany' => __CLASS__,
            ]
        );

        $this->createBaseModelObject(
            'oxuserbaskets',
            $this->aUserBasketIdList[2],
            [
                'oxuserid'    => $this->aUserIdList[2],
                'oxtitle'     => 'savedbasket',
                'oxupdate'     => strtotime(date('Y-m-d') . ' -1 year - 1 minute'),
            ]
        );

        $this->createUser(
            $this->aUserIdList[3],
            [
                'oxcompany' => __CLASS__,
            ]
        );

        $this->createBaseModelObject(
            'oxuserbaskets',
            $this->aUserBasketIdList[3],
            [
                'oxuserid'    => $this->aUserIdList[3],
                'oxtitle'     => 'wishlist',
            ]
        );

        $this->createUser(
            $this->aUserIdList[4],
            [
                'oxcompany' => __CLASS__,
            ]
        );
    }

    public function cleanTestData()
    {
        $this->deleteManager($this->sManagerId);

        foreach ($this->aUserIdList as $sUserId) {
            $this->deleteUser($sUserId);
        }

        foreach ($this->aUserBasketIdList as $sUserBasketId) {
            $this->deleteBaseModelObject('oxuserbaskets', $sUserBasketId);
        }
    }

    /**
     * @return d3usermanager
     * @throws Exception
     */
    public function getConfiguredManager()
    {
        $oManager = $this->getManagerMock($this->sManagerId);

        $oManager->setValue('blCheckUserBasketFrom_status', true);
        $oManager->setValue('sUserBasketFromValue', date('Y-m-d', strtotime(date('Y-m-d') . ' -1 year')));

        return $oManager;
    }

    /**
     * @test
     * @throws DatabaseConnectionException
     * @throws DatabaseErrorException
     * @throws StandardException
     * @throws d3ShopCompatibilityAdapterException
     * @throws d3_cfg_mod_exception
     * @throws Exception
     */
    public function requirementsSelectsRightUsers()
    {
        $oListGenerator = $this->getListGenerator($this->getConfiguredManager());
        $oUserList = $oListGenerator->getConcernedItems();

        $this->assertTrue(
            $oUserList->count() >= 2
            && $oUserList->offsetExists($this->aUserIdList[0])
            && $oUserList->offsetExists($this->aUserIdList[1])
            && false == $oUserList->offsetExists($this->aUserIdList[2])
            && false == $oUserList->offsetExists($this->aUserIdList[3])
            && false == $oUserList->offsetExists($this->aUserIdList[4])
        );
    }
}
